@@ -702,9 +702,27 @@ class PatchTest(TestCase):
         self.assertEqual(1, m.FOO)
 
 
-
-
 class TestdataTest(TestCase):
+    def test_get_phone(self):
+        ph = testdata.get_phone()
+        self.assertRegex(ph, r"\d{3}-\d{3}-\d{4}")
+
+        ph = testdata.get_phone("+{country_code}-{area_code}-{exchange_code}-{line_number}")
+        self.assertRegex(ph, r"\+\d-\d{3}-\d{3}-\d{4}")
+
+        ph = testdata.get_phone(line_number="5555")
+        self.assertTrue(ph.endswith("-5555"))
+
+    def test_get_usa_address(self):
+        a = testdata.get_usa_address()
+        self.assertEqual(a[0], a.street)
+        self.assertEqual(a[1], a.section)
+        self.assertEqual(a[2], a.city)
+        self.assertEqual(a[3], a.state)
+        self.assertEqual(a[4], a.zipcode)
+        self.assertEqual(a[5], a.line)
+        self.assertEqual(a[6], a.lines)
+
     def test_get_filename(self):
         n = testdata.get_filename(ext="py", name="foo")
         self.assertEqual("foo.py", n)
